<html>
    <head>
    </head>
    <body>
    	<h1>Olá, {{ name }}!</h1>
      <p>
        Obrigado pelo seu registo.
      </p>
      <p>
        Para activar a sua conta, precisamos apenas que confirme o seu email clicando
        <a href="{{ activation_link }}">aqui</a>.
      </p>
    </body>
</html>