{% load i18n %}

<h1>{% trans "Account" %}</h1>

<!-- Messages -->
{% if messages %}
  <ul>
  {% for message in messages %}
    [{{ message.tags }}] {{ message }}
  {% endfor %}
  </ul>
{% endif %}

<p>
	<a href="{% url 'accounts:logout' %}">{% trans "Logout" %}</a>
</p>